'use strict';

module.exports = function (opts) {
  var finalhandler = require('finalhandler');
  var serveStatic = require('serve-static');
  var serveIndex = require('serve-index');
  var assetServer = serveStatic(opts.assetsPath);
  var path = require('path');
  var wellKnownServer = serveStatic(path.join(opts.assetsPath, 'well-known'));

  var hostsMap = {};
  var pathsMap = {};
  var content = opts.content;
  var server;
  var serveInit;

  function addServer(hostname) {

    if (hostsMap[hostname]) {
      return hostsMap[hostname];
    }

    opts.sites.forEach(function (site) {

      if (hostname !== site.name) {
        return;
      }

      // path should exist before it gets to this point
      site.path = site.path || site.paths[0];

      if (!pathsMap[site.path]) {
        console.log('site:', site);
        pathsMap[site.path] = {
          serve: serveStatic(site.path)
        // TODO option for dotfiles
        , index: serveIndex(site.path)
        };
      }

      hostsMap[hostname] = {
        serve: pathsMap[site.path].serve
      , index: pathsMap[site.path].index
      , app: site.app
      };

    });

  }

  function _reloadWrite(data, enc, cb) {
    /*jshint validthis: true */
    if (this.headersSent) {
      this.__write(data, enc, cb);
      return;
    }

    if (!/html/i.test(this.getHeader('Content-Type'))) {
      this.__write(data, enc, cb);
      return;
    }

    if (this.getHeader('Content-Length')) {
      this.setHeader('Content-Length', this.getHeader('Content-Length') + this.__my_addLen);
    }

    this.__write(this.__my_livereload);
    this.__write(data, enc, cb);
  }


  addServer(opts.sites[0].name);

  return function (req, res) {
    if ('/api/com.daplie.caddy/init' === req.url) {
      if (!serveInit) {
        serveInit = require('../packages/apis/com.daplie.caddy').create(opts);
      }
      serveInit(req, res);
      return;
    }

    if (content && '/' === req.url) {
      // res.setHeader('Content-Type', 'application/octet-stream');
      res.end(content);
      return;
    }
    var done = finalhandler(req, res);
    var host = req.headers.host;
    var hostname = (host||'').split(':')[0] || opts.sites[0].name;

    function serveStaticly(server) {
      function serveTheStatic() {
        server.serve(req, res, function (err) {
          if (err) { return done(err); }
          server.index(req, res, function (err) {
            if (err) { return done(err); }
            req.url = req.url.replace(/\/assets/, '');
            assetServer(req, res, function  () {
              if (err) { return done(err); }
              req.url = req.url.replace(/\/\.well-known/, '');
              wellKnownServer(req, res, done);
            });
          });
        });
      }

      if (server.expressApp) {
        server.expressApp(req, res, serveTheStatic);
        return;
      }

      serveTheStatic();
    }

    if (opts.livereload) {
      res.__my_livereload = '<script src="//'
        + (host || opts.sites[0].name).split(':')[0]
        + ':35729/livereload.js?snipver=1"></script>';
      res.__my_addLen = res.__my_livereload.length;

      // TODO modify prototype instead of each instance?
      res.__write = res.write;
      res.write = _reloadWrite;
    }

    console.log('hostname:', hostname, opts.sites[0].paths);

    addServer(hostname);
    server = hostsMap[hostname] || hostsMap[opts.sites[0].name];
    serveStaticly(server);

  };
};
